@@ -2,6 +2,8 @@
     begin                : Sat Feb 16 2002
     copyright            : (C) 2002 - 2004 by Scott Wheeler
     email                : wheeler@kde.org
+    copyright            : (c) 2007 Michael Pyne
+    email                : michael.pyne@kdemail.net
 ***************************************************************************/
 
 /***************************************************************************
@@ -16,45 +18,47 @@
 #ifndef PLAYLIST_H
 #define PLAYLIST_H
 
-#include <klistview.h>
-#include <kurldrag.h>
-#include <kdebug.h>
+#include <k3listview.h>
 #include <kglobalsettings.h>
+#include <kdebug.h>
 
-#include <qvaluevector.h>
-#include <qfileinfo.h>
+#include <QVector>
+#include <QEvent>
+#include <QList>
 
 #include "covermanager.h"
 #include "stringhash.h"
 #include "playlistsearch.h"
 #include "tagguesser.h"
 #include "playlistinterface.h"
-#include "playlistitem.h"
 
-class KPopupMenu;
+class KMenu;
 class KActionMenu;
 
-class QEvent;
+class QFileInfo;
+class QMimeData;
+class QDrag;
+class QAction;
 
-class PlaylistCollection;
 class WebImageFetcher;
+class PlaylistItem;
+class PlaylistCollection;
 class PlaylistToolTip;
-class UpcomingPlaylist;
 
-typedef QValueList<PlaylistItem *> PlaylistItemList;
+typedef QList<PlaylistItem *> PlaylistItemList;
 
-class Playlist : public KListView, public PlaylistInterface
+class Playlist : public K3ListView, public PlaylistInterface
 {
     Q_OBJECT
 
 public:
 
-    Playlist(PlaylistCollection *collection, const QString &name = QString::null,
-	     const QString &iconName = "midi");
+    explicit Playlist(PlaylistCollection *collection, const QString &name = QString(),
+             const QString &iconName = "audio-midi");
     Playlist(PlaylistCollection *collection, const PlaylistItemList &items,
-	     const QString &name = QString::null, const QString &iconName = "midi");
+             const QString &name = QString(), const QString &iconName = "audio-midi");
     Playlist(PlaylistCollection *collection, const QFileInfo &playlistFile,
-	     const QString &iconName = "midi");
+             const QString &iconName = "audio-midi");
 
     /**
      * This constructor should generally only be used either by the cache
@@ -162,8 +166,8 @@ public:
      * both PlaylistItems and CollectionListItems.
      */
     virtual PlaylistItem *createItem(const FileHandle &file,
-				     QListViewItem *after = 0,
-				     bool emitChanged = true);
+                                     Q3ListViewItem *after = 0,
+                                     bool emitChanged = true);
 
     /**
      * This is implemented as a template method to allow subclasses to
@@ -174,8 +178,8 @@ public:
      */
     template <class ItemType, class CollectionItemType, class CollectionListType>
     ItemType *createItem(const FileHandle &file,
-			 QListViewItem *after = 0,
-			 bool emitChanged = true);
+                         Q3ListViewItem *after = 0,
+                         bool emitChanged = true);
 
     virtual void createItems(const PlaylistItemList &siblings, PlaylistItem *after = 0);
 
@@ -192,7 +196,7 @@ public:
 
     /**
      * Returns the file name associated with this playlist (an m3u file) or
-     * QString::null if no such file exists.
+     * an empty QString if no such file exists.
      */
     QString fileName() const { return m_fileName; }
 
@@ -296,7 +300,7 @@ public:
     /**
      * Returns true if it's possible to reload this playlist.
      */
-    virtual bool canReload() const { return !m_fileName.isNull(); }
+    virtual bool canReload() const { return !m_fileName.isEmpty(); }
 
     /**
      * Returns true if the playlist is a search playlist and the search should be
@@ -326,7 +330,7 @@ public slots:
     /**
      * Remove the currently selected items from the playlist and disk.
      */
-    void slotRemoveSelectedItems() { removeFromDisk(selectedItems()); };
+    void slotRemoveSelectedItems() { removeFromDisk(selectedItems()); }
 
     /*
      * The edit slots are required to use the canonical names so that they are
@@ -352,7 +356,7 @@ public slots:
      * @see clearItems()
      */
     virtual void clear();
-    virtual void selectAll() { KListView::selectAll(true); }
+    virtual void selectAll() { K3ListView::selectAll(true); }
 
     /**
      * Refreshes the tags of the selection from disk, or all of the files in the
@@ -419,10 +423,9 @@ protected:
 
     virtual bool eventFilter(QObject *watched, QEvent *e);
     virtual void keyPressEvent(QKeyEvent *e);
-    virtual QDragObject *dragObject(QWidget *parent);
-    virtual QDragObject *dragObject() { return dragObject(this); }
-    virtual bool canDecode(QMimeSource *s);
-    virtual void decode(QMimeSource *s, PlaylistItem *item = 0);
+    virtual Q3DragObject *dragObject(QWidget *parent);
+    virtual Q3DragObject *dragObject() { return dragObject(this); }
+    virtual void decode(const QMimeData *s, PlaylistItem *item = 0);
     virtual void contentsDropEvent(QDropEvent *e);
     virtual void contentsMouseDoubleClickEvent(QMouseEvent *e);
     virtual void contentsDragEnterEvent(QDragEnterEvent *e);
@@ -431,21 +434,13 @@ protected:
     virtual void viewportPaintEvent(QPaintEvent *pe);
     virtual void viewportResizeEvent(QResizeEvent *re);
 
-    virtual void insertItem(QListViewItem *item);
-    virtual void takeItem(QListViewItem *item);
+    virtual void insertItem(Q3ListViewItem *item);
+    virtual void takeItem(Q3ListViewItem *item);
 
     virtual bool hasItem(const QString &file) const { return m_members.contains(file); }
 
-    void addColumn(const QString &label);
-
-    /**
-     * Here I'm using delayed setup of some things that aren't quite intuitive.
-     * Creating columns and setting up connections are both time consuming if
-     * there are a lot of playlists to initialize.  This moves that cost from the
-     * startup time to the time when the widget is "polished" -- i.e. just before
-     * it's painted the first time.
-     */
-    virtual void polish();
+    virtual int addColumn(const QString &label, int width = -1);
+    using K3ListView::addColumn;
 
     /**
      * Do some finial initialization of created items.  Notably ensure that they
@@ -470,11 +465,11 @@ protected:
      * ItemType should be a PlaylistItem subclass.
      */
     template <class CollectionItemType, class ItemType, class SiblingType>
-    void createItems(const QValueList<SiblingType *> &siblings, ItemType *after = 0);
+    void createItems(const QList<SiblingType *> &siblings, ItemType *after = 0);
 
 protected slots:
     void slotPopulateBackMenu() const;
-    void slotPlayFromBackMenu(int number) const;
+    void slotPlayFromBackMenu(QAction *) const;
 
 signals:
 
@@ -536,7 +531,7 @@ private:
      * \see visibleItems()
      * \see selectedItems()
      */
-    PlaylistItemList items(QListViewItemIterator::IteratorFlag flags);
+    PlaylistItemList items(Q3ListViewItemIterator::IteratorFlag flags);
 
     /**
      * Build the column "weights" for the weighted width mode.
@@ -544,9 +539,9 @@ private:
     void calculateColumnWeights();
 
     void addFile(const QString &file, FileHandleList &files, bool importPlaylists,
-		 PlaylistItem **after);
+                 PlaylistItem **after);
     void addFileHelper(FileHandleList &files, PlaylistItem **after,
-		       bool ignoreTimer = false);
+                       bool ignoreTimer = false);
 
     void redisplaySearch() { setSearch(m_search); }
 
@@ -557,13 +552,6 @@ private:
 
     void refreshAlbum(const QString &artist, const QString &album);
 
-    /**
-     * Returns the number of PlaylistItems in @p items that can be assigned a
-     * cover.  Used to avoid wasting the users' time setting the cover for 20
-     * items when none are eligible.
-     */
-    unsigned int eligibleCoverItems(const PlaylistItemList &items);
-
     void updatePlaying() const;
 
     /**
@@ -572,17 +560,29 @@ private:
      */
     class SharedSettings;
 
+    using K3ListView::selectAll; // Avoid warning about hiding this function.
+
 private slots:
 
+    /**
+     * Handle the necessary tasks needed to create and setup the playlist that
+     * don't need to happen in the ctor, such as setting up the columns,
+     * initializing the RMB menu, and setting up signal/slot connections.
+     *
+     * Used to be a subclass of K3ListView::polish() but the timing of the
+     * call is not consistent and therefore lead to crashes.
+     */
+    void slotInitialize();
+
     void slotUpdateColumnWidths();
 
     void slotAddToUpcoming();
 
     /**
-     * Show the RMB menu.  Matches the signature for the signal 
+     * Show the RMB menu.  Matches the signature for the signal
      * QListView::contextMenuRequested().
      */
-    void slotShowRMBMenu(QListViewItem *item, const QPoint &point, int column);
+    void slotShowRMBMenu(Q3ListViewItem *item, const QPoint &point, int column);
 
     /**
      * This slot is called when the inline tag editor has completed its editing
@@ -590,10 +590,10 @@ private slots:
      *
      * \see editTag()
      */
-    void slotInlineEditDone(QListViewItem *, const QString &, int column);
+    void slotInlineEditDone(Q3ListViewItem *, const QString &, int column);
 
     /**
-     * This starts the renaming process by displaying a line edit if the mouse is in 
+     * This starts the renaming process by displaying a line edit if the mouse is in
      * an appropriate position.
      */
     void slotRenameTag();
@@ -616,7 +616,7 @@ private slots:
      * \see hideColumn()
      * \see showColumn()
      */
-    void slotToggleColumnVisible(int column);
+    void slotToggleColumnVisible(QAction *action);
 
     /**
      * Prompts the user to create a new playlist with from the selected items.
@@ -648,16 +648,14 @@ private:
 
     int m_currentColumn;
     int m_processed;
-    int m_rmbEditID;
-    int m_rmbUpcomingID;
+    QAction *m_rmbEdit;
     int m_selectedCount;
 
     bool m_allowDuplicates;
-    bool m_polished;
     bool m_applySharedSettings;
     bool m_columnWidthModeChanged;
 
-    QValueList<int> m_weightDirty;
+    QList<int> m_weightDirty;
     bool m_disableColumnWidthUpdates;
 
     mutable int m_time;
@@ -667,8 +665,8 @@ private:
     /**
      * The average minimum widths of columns to be used in balancing calculations.
      */
-    QValueVector<int> m_columnWeights;
-    QValueVector<int> m_columnFixedWidths;
+    QVector<int> m_columnWeights;
+    QVector<int> m_columnFixedWidths;
     bool m_widthsDirty;
 
     static PlaylistItemList m_history;
@@ -692,8 +690,8 @@ private:
     QString m_playlistName;
     QString m_fileName;
 
-    KPopupMenu *m_rmbMenu;
-    KPopupMenu *m_headerMenu;
+    KMenu *m_rmbMenu;
+    KMenu *m_headerMenu;
     KActionMenu *m_columnVisibleAction;
     PlaylistToolTip *m_toolTip;
 
@@ -709,13 +707,17 @@ private:
     bool m_blockDataChanged;
 };
 
+typedef QList<Playlist *> PlaylistList;
+
 bool processEvents();
 
-class FocusUpEvent : public QCustomEvent
+class FocusUpEvent : public QEvent
 {
 public:
-    FocusUpEvent() : QCustomEvent(id) {}
-    static const int id = 999;
+    FocusUpEvent() : QEvent(id) {}
+    Type type() const { return id; }
+
+    static const Type id = static_cast<Type>(QEvent::User + 1);
 };
 
 QDataStream &operator<<(QDataStream &s, const Playlist &p);
@@ -724,47 +726,47 @@ QDataStream &operator>>(QDataStream &s, Playlist &p);
 // template method implementations
 
 template <class ItemType, class CollectionItemType, class CollectionListType>
-ItemType *Playlist::createItem(const FileHandle &file, QListViewItem *after,
-			       bool emitChanged)
+ItemType *Playlist::createItem(const FileHandle &file, Q3ListViewItem *after,
+                               bool emitChanged)
 {
     CollectionItemType *item = CollectionListType::instance()->lookup(file.absFilePath());
 
     if(!item) {
-	item = new CollectionItemType(file);
-	setupItem(item);
+        item = new CollectionItemType(file);
+        setupItem(item);
 
-	// If a valid tag was not created, destroy the CollectionListItem.
+        // If a valid tag was not created, destroy the CollectionListItem.
 
-	if(!item->isValid()) {
-	    kdError(65432) << "Playlist::createItem() -- A valid tag was not created for \""
-			   << file.absFilePath() << "\"" << endl;
-	    delete item;
-	    return 0;
-	}
+        if(!item->isValid()) {
+            kError(65432) << "Playlist::createItem() -- A valid tag was not created for \""
+                           << file.absFilePath() << "\"" << endl;
+            delete item;
+            return 0;
+        }
     }
 
     if(item && !m_members.insert(file.absFilePath()) || m_allowDuplicates) {
 
-	ItemType *i = after ? new ItemType(item, this, after) : new ItemType(item, this);
-	setupItem(i);
+        ItemType *i = after ? new ItemType(item, this, after) : new ItemType(item, this);
+        setupItem(i);
 
-	if(emitChanged)
-	    dataChanged();
+        if(emitChanged)
+            dataChanged();
 
-	return i;
+        return i;
     }
     else
-	return 0;
+        return 0;
 }
 
 template <class ItemType, class SiblingType>
 ItemType *Playlist::createItem(SiblingType *sibling, ItemType *after)
 {
     m_disableColumnWidthUpdates = true;
-    
+
     if(!m_members.insert(sibling->file().absFilePath()) || m_allowDuplicates) {
-	after = new ItemType(sibling->collectionItem(), this, after);
-	setupItem(after);
+        after = new ItemType(sibling->collectionItem(), this, after);
+        setupItem(after);
     }
 
     m_disableColumnWidthUpdates = false;
@@ -773,17 +775,18 @@ ItemType *Playlist::createItem(SiblingType *sibling, ItemType *after)
 }
 
 template <class CollectionItemType, class ItemType, class SiblingType>
-void Playlist::createItems(const QValueList<SiblingType *> &siblings, ItemType *after)
+void Playlist::createItems(const QList<SiblingType *> &siblings, ItemType *after)
 {
     if(siblings.isEmpty())
-	return;
+        return;
 
-    QValueListConstIterator<SiblingType *> it = siblings.begin();
-    for(; it != siblings.end(); ++it)
-	after = createItem(*it, after);
+    foreach(SiblingType *sibling, siblings)
+        after = createItem(sibling, after);
 
     dataChanged();
     slotWeightDirty();
 }
 
 #endif
+
+// vim: set et sw=4 tw=0 sta:
