@@ -1,6 +1,7 @@
 /*
     Copyright (C) 2004 Allan Sandfeld Jensen <kde@carewolf.com>
-
+    Copyright (C) 2005 Benjamin Meyer <ben at meyerhome dot net>
+ 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
     License as published by the Free Software Foundation; either
@@ -13,8 +14,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include "encoderflac.h"
@@ -27,15 +28,16 @@
 
 #include <kconfig.h>
 #include <kdebug.h>
-
+#include <QPair>
+#include <QDateTime>
 
 extern "C"
 {
-  KDE_EXPORT void create_audiocd_encoders(KIO::SlaveBase *slave, QPtrList<AudioCDEncoder> &encoders)
+  KDE_EXPORT void create_audiocd_encoders(KIO::SlaveBase *slave, QList<AudioCDEncoder*> &encoders)
   {
     encoders.append(new EncoderFLAC(slave));
   }
-};
+}
 
 class EncoderFLAC::Private {
 
@@ -46,7 +48,16 @@ public:
     unsigned long data;
 };
 
-static FLAC__StreamEncoderWriteStatus WriteCallback(const FLAC__StreamEncoder *encoder, const FLAC__byte buffer[], unsigned bytes, unsigned samples, unsigned current_frame, void *client_data)
+static FLAC__StreamEncoderWriteStatus WriteCallback(const FLAC__StreamEncoder *encoder,
+                                                    const FLAC__byte buffer[],
+#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT <= 7
+                                                    unsigned bytes,
+#else
+                                                    size_t bytes,
+#endif
+                                                    unsigned samples,
+                                                    unsigned current_frame,
+                                                    void *client_data)
 {
     EncoderFLAC::Private *d = (EncoderFLAC::Private*)client_data;
 
@@ -61,13 +72,13 @@ static FLAC__StreamEncoderWriteStatus WriteCallback(const FLAC__StreamEncoder *e
     }
 
     return FLAC__STREAM_ENCODER_WRITE_STATUS_OK;
-};
+}
 
 static void MetadataCallback (const FLAC__StreamEncoder *encoder, const FLAC__StreamMetadata *metadata, void *client_data)
 {
 	// We do not have seekable writeback so we just discard the updated metadata
 
-};
+}
 
 /*
 static FLAC__SeekableStreamEncoderSeekStatus  SeekCallback(const FLAC__SeekableStreamEncoder *encoder,  FLAC__uint64 absolute_byte_offset, void *client_data)
@@ -106,11 +117,13 @@ unsigned long EncoderFLAC::size(long time_secs) const {
 }
 
 long EncoderFLAC::readInit(long size) {
-    kdDebug(7117) << "EncoderFLAC::readInit() called"<< endl;
+    kDebug(7117) << "EncoderFLAC::readInit() called";
     d->data = 0;
+#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT <= 7
     FLAC__stream_encoder_set_write_callback(d->encoder, WriteCallback);
     FLAC__stream_encoder_set_metadata_callback(d->encoder, MetadataCallback);
     FLAC__stream_encoder_set_client_data(d->encoder, d);
+#endif
 
     // The options match approximely those of flac compression-level-3
     FLAC__stream_encoder_set_do_mid_side_stereo(d->encoder, true);
@@ -123,7 +136,11 @@ long EncoderFLAC::readInit(long size) {
     if (size > 0)
         FLAC__stream_encoder_set_total_samples_estimate(d->encoder, size/4);
 
+#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT <= 7
     FLAC__stream_encoder_init(d->encoder);
+#else
+    FLAC__stream_encoder_init_stream(d->encoder, WriteCallback, NULL, NULL, MetadataCallback, d);
+#endif
     return d->data;
 }
 
@@ -152,36 +169,34 @@ long EncoderFLAC::readCleanup()
     return 0;
 }
 
-void EncoderFLAC::fillSongInfo(const QString &trackName, const QString &cdArtist,
-			       const QString &cdTitle,   const QString &cdCategory,
-			       int trackNumber, int cdYear, const QString &comment)
+void EncoderFLAC::fillSongInfo( KCDDB::CDInfo info, int track, const QString &comment )
 {
     d->metadata = new FLAC__StreamMetadata*[1];
     d->metadata[0] = FLAC__metadata_object_new(FLAC__METADATA_TYPE_VORBIS_COMMENT);
 //    d->metadata[1] = FLAC__metadata_object_new(FLAC__METADATA_TYPE_PADDING);
 //    d->metadata[2] = FLAC__metadata_object_new(FLAC__METADATA_TYPE_SEEKTABLE)
 
-    typedef QPair<QString, QString> Comment;
-    Comment comments[7] = { Comment("Title", trackName),
-	    	    	    Comment("Artist", cdArtist),
-	    	    	    Comment("Album",  cdTitle),
-	    	    	    Comment("Genre",  cdCategory),
-	    	    	    Comment("Tracknumber", QString::number(trackNumber)),
-									Comment("Comment", comment),
-	    	    	    Comment("Date", QString::null )};
-    if (cdYear > 0) {
-    	QDateTime dt = QDate(cdYear, 1, 1);
+    typedef QPair<QString, QVariant> Comment;
+    Comment comments[7] = { Comment("Title", info.track(track).get(Title)),
+	    	    	    Comment("Artist", info.track(track).get(Artist)),
+	    	    	    Comment("Album",  info.get(Title)),
+	    	    	    Comment("Genre",  info.get(Genre)),
+	    	    	    Comment("Tracknumber", QString::number(track)),
+	                Comment("Comment", comment),
+	    	    	    Comment("Date", QVariant(QString::null) )};	//krazy:exclude=nullstrassign for old broken gcc
+    if (info.get(Year).toInt() > 0) {
+    	QDateTime dt(QDate(info.get(Year).toInt(), 1, 1));
     	comments[6] = Comment("Date", dt.toString(Qt::ISODate));
     }
 
     FLAC__StreamMetadata_VorbisComment_Entry entry;
     QString field;
-    QCString cfield;
+    QByteArray cfield;
     int num_comments = 0;
 
     for(int i=0; i<7; i++) {
-	if (!comments[i].second.isEmpty()) {
-	    field = comments[i].first+"="+comments[i].second;
+	if (!comments[i].second.toString().isEmpty()) {
+	    field = comments[i].first+'='+comments[i].second.toString();
 	    cfield = field.utf8();
     	    entry.entry = (FLAC__byte*)qstrdup(cfield);
 	    entry.length = cfield.length();
@@ -192,7 +207,6 @@ void EncoderFLAC::fillSongInfo(const QString &trackName, const QString &cdArtist
     }
 
     FLAC__stream_encoder_set_metadata(d->encoder, d->metadata, 1);
-
 }
 
 #endif // HAVE_LIBFLAC
